#include "stm32f030xx.h"
/* Test program for the STM32F030 RTC Alarm interrupt
 * The idea is that the RTC interrupts the CPU about once per second
 * Inside the RTC ISR PC8 is toggled
 * Inside the main loop (when the CPU comes out of stop) PC9 is toggled
 * Timing is based on the LSI oscillator and is approximate
 * 
*/
void RTCISR(void)
{
	RTC_ISR &= ~BIT8;
	EXTI_PR |= BIT17;
	GPIOC_ODR ^= BIT8;
	
}
void initRTC()
{	
	// Turn on power control circuit clock
	RCC_APB1ENR |= BIT28;
	// Turn on LSI
	RCC_CSR |= BIT0;
	// Wait for LSI to be ready
	while ( (RCC_CSR & BIT1) == 0);
	// Unlock the RTC domain
	PWR_CR |= BIT8; // set DBP in PWR_CR
	RCC_BDCR |=  BIT16; // put RTC power domain into reset
	RCC_BDCR &=  ~BIT16; // take it back out of reset
	
	// Turn the RTC on, use LSI
	RCC_BDCR |= BIT15 + BIT9;
	RCC_BDCR &= ~BIT8;
	// Unlock the RTC
	RTC_WPR = 0xca;
	RTC_WPR = 0x53;
	// RTC Initialization procedure (see reference manual)
	RTC_ISR |= BIT7; // set INIT bit
	while ((RTC_ISR & BIT6)==0); // wait for init to start
	
	ISER |= BIT2;   // enable RTC IRQ in NVIC
	
// Alarm interrupt configuration	
	RTC_CR = 0;
	RTC_ALRMAR = BIT31+BIT23+BIT15+BIT7; // ignore all fields -> alarm every second (approx)
	RTC_ALRMASSR = 0x0f000000;  // match all sub second bits
	RTC_CR |= BIT12+BIT8;		// Enable alarm and alarm interrupt
	RTC_ISR &= ~BIT7; 			// clear INIT bit
	
// RTC alarm is an EXTI interrupt in fact so need to configure this too
	EXTI_IMR |= BIT17;  // RTC is triggered via EXTI17,18 or 19
	EXTI_RTSR |= BIT17;
}
void ConfigPins()
{
	RCC_AHBENR |= BIT19; // turn on port C
	GPIOC_MODER &= ~(BIT17+BIT27+BIT26);
	GPIOC_MODER |= (BIT16+BIT18);	
}
void delay(uint32_t dly)
{
	while(dly--);
}
int main()
{	
	ConfigPins();
	initRTC();
	enable_interrupts();
	GPIOC_ODR = 0;//= BIT8;
	SCR  |= BIT2; // enable deep sleep
	while(1)
	{
		GPIOC_ODR ^= BIT9;
		cpu_sleep();					
	}
}
